<?php

class SebPaymentGatewayLibrary
{
    const _VERIFY_SUCCESS = 1; // payment successful
    const _VERIFY_CANCEL = 2;  // payment cancelled
    const _VERIFY_FAIL = 3;    // payment failed

    private $api_username;
    private $api_secret;
    private $request_cc_token;
    private $cc_token;
    private $statuses = array(
        'completed' => self::_VERIFY_SUCCESS,
        'cancelled' => self::_VERIFY_CANCEL,
        'failed'    => self::_VERIFY_FAIL
    );

    /**
      * Initiates a payment.
      *
      * Expects following data as input:
      * 'api_username' => api username,
      * 'api_secret => api secret,
      * array(
      *  'request_cc_token' => request to get cc_token 1,
      *  'cc_token' => token referencing a bank card
      * )
      *
      * @param $api_username string
      * @param $api_secret string
      * @param $data array
      * @example $sebpaymentgateway->init('api_username', 'api_secret', array('cc_token' => 'token'));
      *
    */

    public function init($api_username, $api_secret, $data = '')
    {
        $this->api_username = $api_username;
        $this->api_secret = $api_secret;

        if (isset($data['request_cc_token'])) {
          $this->request_cc_token = $data['request_cc_token'];
        }

        if (isset($data['cc_token'])) {
          $this->cc_token = $data['cc_token'];
        }
    }

    /**
      * Populates and returns array of fields to be submitted for payment.
      *
      * Expects following data as input:
      * array(
      *  'account_id' => account id,
      *  'amount' => amount to pay,
      *  'billing_address' => billing address street name,
      *  'billing_city' => billing address city name,
      *  'billing_country' => billing address 2 letter country code,
      *  'billing_postcode' => billing address postal code,
      *  'callback_url' => callback url,
      *  'customer_url' => return url,
      *  'delivery_address' => shipping address street name,
      *  'delivery_city' => shipping address city name,
      *  'delivery_country' => shipping address 2 letter country code,
      *  'delivery_postcode' => shipping address postal code,
      *  'email' => customer email address,
      *  'order_reference' => order reference number,
      *  'user_ip' => user ip address,
      *  'hmac_fields' => string which contains all fields (keys) that are going to be used in hmac calculation, separated by comma,
      *  'request_cc_token' => request to get cc_token 1,
      *  'cc_token' => token referencing a bank card
      * );
      *
      * @param $data array
      * @param $language string
      *
      * @return array
    */

    public function getFields(array $data, $language = 'en')
    {
        $data['api_username'] = $this->api_username;
        $data['nonce'] = $this->getNonce();
        $data['timestamp'] = date('c');
        $data['transaction_type'] = 'charge';

        if (isset($this->request_cc_token)) {
            $data['request_cc_token'] = $this->request_cc_token;
        }

        if (isset($this->cc_token)) {
            $data['cc_token'] = $this->cc_token;
        }

        /*if (isset($data['hmac_fields'])) {
            $keys = array_keys($data);
            $keys[] = 'hmac_fields';
            asort($keys);
            $data['hmac_fields'] = implode(',', $keys);
        }*/

        //$data['hmac'] = $this->signData($this->serializeData($data));
        $data['locale'] = $language;
        $module = Module::getInstanceByName("sebpaymentgateway");
        $data["integration_details"] = $this->getIntegrationDetails(
            Configuration::get("PS_SHOP_NAME"),
            Configuration::get("PS_INSTALL_VERSION"),
            $module->name . " " . $module->version . " module"
        );
        return $data;
    }

    public function getIntegrationDetails($softwareName, $version, $integration) {
        return array(
            "software" => $softwareName,
            "version" => $version,
            "integration" => $integration
        );
    }

    /**
      * Verifies return data
      *
      * Expects following data as input:
      *
      * for successful and failed payments:
      *
      * array(
      *  'account_id' => account id in Everypay system,
      *  'amount' => amount to pay,
      *  'api_username' => api username,
      *  'nonce' => return nonce,
      *  'order_reference' => order reference number,
      *  'payment_reference' => payment reference number,
      *  'payment_state' => payment state,
      *  'timestamp' => timestamp,
      *  'transaction_result' => transaction result,
      *  'hmac_fields' => string which contains all fields (keys) that are going to be used in hmac calculation, separated by comma.
      * );
      *
      * for cancelled payments:
      *
      * array(
      *  'api_username' => api username,
      *  'nonce' => return nonce,
      *  'order_reference' => order reference number,
      *  'payment_state' => payment state,
      *  'timestamp' => timestamp,
      *  'transaction_result' => transaction result
      * );
      *

      * @param $data array
      *
      * @return int 1 - verified successful payment, 2 - cancelled, 3 - failed
      * @throws Exception
    */

    public function verify(array $data)
    {
        if ($data['api_username'] !== $this->api_username) {
            throw new Exception('Invalid username.');
        }

        $now = time();
        if (($data['timestamp'] > $now) || ($data['timestamp'] < ($now - 600))) {
            throw new Exception('Response outdated.');
        }

        /**
          * Refer to the Integration Manual for more information about 'order_reference' validation.
        */

        if (!$this->verifyNonce($data['nonce'])) {
            throw new Exception('Nonce is already used.');
        }

        /**
          * Refer to the Integration Manual for more information about 'nonce' uniqueness validation.
        */

        $status = $this->statuses[$data['transaction_result']];

        $verify = array();
        $hmac_fields = explode(',', $data["hmac_fields"]);

        foreach ($hmac_fields as $value) {
            $verify[$value] = isset($data[$value]) ? $data[$value] : '';
        }

        $hmac = $this->signData($this->serializeData($verify));
        if ($data['hmac'] != $hmac) {
            throw new Exception('Invalid HMAC.');
        }

        return $status;
    }

    protected function getNonce()
    {
        return uniqid(true);
    }

    protected function verifyNonce($nonce)
    {
        return true;
    }

    /**
      * Prepare data package for signing
      *
      * @param array $data
      * @return string
    */

    private function serializeData(array $data)
    {
        $arr = array();
        ksort($data);
        foreach ($data as $k => $v) {
            $arr[] = $k . '=' . $v;
        }
        return implode('&', $arr);
    }

    private function signData($data)
    {
        return hash_hmac('sha1', $data, $this->api_secret);
    }

    public function getAPIUsername() {
        return $this->api_username;
    }

    public function getAPISecret() {
        return $this->api_secret;
    }
}
